//========================================================
//        File: fileio.c
//
// Description: Function for reading and writing the key
//              configuration files.
//
//          By: Kurt Hoffmann
//        Date: 3/3/2002
//     Version: 1.0
//========================================================


#include "fileio.h"


//========================================================
//    Function: OpenFile
//
// Description: Reads in a key configuration file.
//
//  Parameters: const char*     - the name of the file
//              char**          - list of key commands
//              unsigned short* - list of row numbers
//              unsigned short* - list of column numbers
//
//     Returns: int - the number of key commands read
//                    or -1 if there was a problem
//========================================================
int OpenFile(const char *filename,
             char **key_list,
             unsigned short *row_list,
             unsigned short *col_list)
{
  char keyname[KEYNAME_LENGTH+1];
  unsigned short row = 0;
  unsigned short col = 0;
  int success = -1;
  int count = 0;
  FILE *fp = NULL;

  if ((fp = fopen(filename, "rb")) != NULL)
  {
  	// read in the key configurator identification
    fread(keyname, sizeof(char), 3, fp);
    if (strncmp(keyname, "kc1", 3) == 0)
    {

      // read in the info
      while (ReadKey(fp, keyname, &row, &col) == TRUE)
      {
        // add keyname, row, and column to the list
        key_list[count] = malloc(sizeof(char)*(KEYNAME_LENGTH+1));
        memset(key_list[count], '\0', (KEYNAME_LENGTH+1));
        strncpy(key_list[count], keyname, KEYNAME_LENGTH);

        row_list[count] = row;
        col_list[count] = col;

        count++;
      }
      fclose(fp);
      success = count;
    }
  }

  return success;
}


//========================================================
//    Function: ReadKey
//
// Description: Reads one key command from a file
//
//  Parameters: FILE*           - the handle of the file
//              char*           - store the command name
//              unsigned short* - store the row number
//              unsigned short* - store the column number
//
//     Returns: BOOL - was a key successfully read?
//========================================================
BOOL ReadKey(FILE *fp,
             char *keyname,
             unsigned short *row,
             unsigned short *column)
{
  BOOL success = TRUE;

  if(fp != NULL)
  {
    // key name
    if(fread(keyname, sizeof(char), KEYNAME_LENGTH, fp) != KEYNAME_LENGTH)
    {
    	success = FALSE;
    }

    // key row
    if(success == TRUE && fread(row, sizeof(unsigned short), 1, fp) != 1)
    {
    	success = FALSE;
    }

    // key column
    if(success == TRUE && fread(column, sizeof(unsigned short), 1, fp) != 1)
    {
    	success = FALSE;
    }
	}

	return success;
}


//========================================================
//    Function: WriteKey
//
// Description: Writes one key command to a file.
//
//  Parameters: FILE*           - the handle of the file
//              char*           - the command name
//              unsigned short* - the row number
//              unsigned short* - the column number
//
//     Returns: None
//========================================================
void WriteKey(FILE *fp,
              char *keyname,
              unsigned short row,
              unsigned short column)
{
  if (fp != NULL)
  {
	  fwrite(keyname, sizeof(char), KEYNAME_LENGTH, fp);
    fwrite(&row, sizeof(unsigned short), 1, fp);
    fwrite(&column, sizeof(unsigned short), 1, fp);
  }
}


//========================================================
//    Function: WriteFile
//
// Description: Writes an entire key configuration file.
//
//  Parameters: const char*     - name of the config file
//              int             - number of key commands
//              char**          - the command names
//              unsigned short* - the row numbers
//              unsigned short* - the column numbers
//
//     Returns: BOOL - was the file successfully written?
//========================================================
BOOL WriteFile(const char* filename,
               int count,
               char **key_list,
               unsigned short *row_list,
               unsigned short *col_list)
{
  FILE *fp = NULL;
  BOOL success = FALSE;
  int loop = 0;

  // unarchive (if archived)
	EM_moveSymFromExtMem(filename, HS_NULL); 

  fp = fopen(filename, "wb");

  if (fp != NULL)
  {
  	// key configuator identification bytes
  	fputs("kc1", fp);

    for (loop = 0; loop < count; loop++)
    {
    	WriteKey(fp, key_list[loop], row_list[loop], col_list[loop]);
    }

    // standard close
    fputc(0, fp);
    fputs("INI", fp);
	  fputc(0, fp);
    fputc(OTH_TAG, fp);
    fclose(fp);

    success = TRUE;
  }

  return success;
}
